/*
unpackpd.idc v1.0 by Dennis Elser
---------------------------------

What does it do?
----------------
This IDC script will automatically unpack packed
pdb symbol files (*.pd_) by invoking the external
utility expand.exe.


Usage
-----
"expand.exe" needs to be copied to your IDA root.
The packed symbol file (.pd_) needs to be saved in
the same directory as the file you are disassembling.

IDA 4.6 and above will invoke your standard web browser
to the corresponding pd_ file.
The script will detect the presence of a pd_ file
and try to unpack it. You can then use IDA's pdb loader
or PDBPlus by Dean Ashton to load the symbols.


How to install the script
-------------------------
unpackpd.idc is meant to be part of IDA's ONLOAD.IDC
script which will automatically be invoked when
opening a new file.

1. copy unpackpd.idc to your ida\idc folder
2. modify onload.idc to include unpackpd.idc (see below)
3. uncomment the code as shown below (#define..., static userload...
4. add "unpackpd(input_file,real_file,filetype);" to the
   userload() function.
5. save and close ONLOAD.IDC
6. be sure that you copied "expand.exe" to your IDA root.


ONLOAD.IDC should look like this after you've modified it:

#include <idc.idc>
#include <unpackpd.idc>

//      If you want to add your own processing of newly created databases,
//      you may create a file named "userload.idc":
//
      #define USERLOAD_IDC
      static userload(input_file,real_file,filetype) {
			unpackpd(input_file,real_file,filetype);
      }

dennis(at)backtrace.de
*/



static unpackpd(input_file,real_file,filetype)
{
	auto pdfile;
	auto expandexe;
	auto pdbfile;
	auto errcode;
	
	pdfile=ReplaceExt(input_file, ".pd_");
	if(!fileexist(pdfile))
	{
		return;
	}
	Message("unpackpd: %s found, invoking expand.exe\n",pdfile);
	
	expandexe=GetIdaDirectory()+"\\expand.exe";
	if(!fileexist(expandexe))
	{
		Message("unpackpd: ERROR! %s not found!\n",expandexe);
		return;
	}
	//Message(expandexe+ " -r " + pdfile+"\n");
	if( (errcode = Exec(expandexe+ " -r " + pdfile)) != 0)
	{
		Message("unpackpd: ERROR! expand.exe returned errorcode %d\n",errcode);
		return;
	}
	
	pdbfile = ReplaceExt(input_file, ".pdb");
	if(fileexist(pdbfile))
	{
		Message("unpackpd: %s created!\nunpackpd: You can load its symbols now!\n",pdbfile);
		return;
	}
	Message("unpackpd: ERROR! %s not available.\n",pdbfile);
	return;
}

static ReplaceExt(filename, extension)
{
	return substr(filename,
			    0,
			    strstr(filename,
			    ".")
			   ) + extension;		
}

static fileexist(filename)
{
	auto fp;

	fp=fopen(filename,"rb");
	if(fp!=0)
	{
		fclose(fp);
		return 1;
	}
	return 0;
}